@@ -6,6 +6,7 @@ import logging
 import os
 
 import pkg_resources
+from dotenv import load_dotenv
 from termcolor import colored
 
 from .benchmarker import Benchmarker  # noqa
@@ -19,6 +20,10 @@ __version__ = pkg_resources.get_distribution("scandeval").version
 block_terminal_output()
 
 
+# Loads environment variables
+load_dotenv()
+
+
 # Set up logging
 fmt = colored("%(asctime)s [%(levelname)s] <%(name)s>\n↳ ", "green") + colored(
     "%(message)s", "yellow"
@@ -28,3 +33,7 @@ logging.basicConfig(level=logging.INFO, format=fmt)
 
 # Disable parallelisation when tokenizing, as that can lead to errors
 os.environ["TOKENIZERS_PARALLELISM"] = "false"
+
+
+# Enable MPS fallback
+os.environ["PYTORCH_ENABLE_MPS_FALLBACK"] = "1"
