@@ -1,6 +1,6 @@
 """Command-line interface for benchmarking."""
 
-from typing import List, Tuple
+from typing import List, Tuple, Union
 
 import click
 
@@ -18,11 +18,10 @@ from .languages import get_all_languages
     show_default=True,
     multiple=True,
     help="""The Hugging Face model ID of the model(s) to be benchmarked. If not
-            specified then all models will be benchmarked, filtered by `model_language`
-            and `model_task`. The specific model version to use can be added after the
-            suffix "@": "<model_id>@v1.0.0". It can be a branch name, a tag name, or a
-            commit id (currently only supported for Hugging Face models, and it defaults
-            to "main" for latest).""",
+    specified then all models will be benchmarked, filtered by `model_language` and
+    `model_task`. The specific model version to use can be added after the suffix "@":
+    "<model_id>@v1.0.0". It can be a branch name, a tag name, or a commit id, and
+    defaults to the latest version if not specified.""",
 )
 @click.option(
     "--dataset",
@@ -32,7 +31,7 @@ from .languages import get_all_languages
     multiple=True,
     type=click.Choice(list(get_all_dataset_configs().keys())),
     help="""The name of the benchmark dataset. If not specified then all datasets will
-            be benchmarked, filtered by `dataset_language` and `dataset_task`.""",
+    be benchmarked, filtered by `dataset_language` and `dataset_task`.""",
 )
 @click.option(
     "--language",
@@ -43,7 +42,8 @@ from .languages import get_all_languages
     metavar="ISO 639-1 LANGUAGE CODE",
     type=click.Choice(["all"] + list(get_all_languages().keys())),
     help="""The languages to benchmark, both for models and datasets. Only relevant if
-            `model-id` and `dataset` have not both been specified.""",
+    `model-id` and `dataset` have not both been specified. If "all" then all models
+    will be benchmarked on all datasets.""",
 )
 @click.option(
     "--model-language",
@@ -54,8 +54,8 @@ from .languages import get_all_languages
     metavar="ISO 639-1 LANGUAGE CODE",
     type=click.Choice(["all"] + list(get_all_languages().keys())),
     help="""The model languages to benchmark. Only relevant if `model-id` has not been
-            specified. If "all" then models will not be filtered according to their
-            language. If not specified then this will use the `language` value.""",
+    specified. If "all" then all models will be benchmarked. If not specified then this
+    will use the `language` value.""",
 )
 @click.option(
     "--dataset-language",
@@ -66,18 +66,8 @@ from .languages import get_all_languages
     metavar="ISO 639-1 LANGUAGE CODE",
     type=click.Choice(["all"] + list(get_all_languages().keys())),
     help="""The dataset languages to benchmark. Only relevant if `dataset` has not been
-            specified. If "all" then datasets will not be filtered according to their
-            language. If not specified then this will use the `language` value.""",
-)
-@click.option(
-    "--model-task",
-    "-mt",
-    default=None,
-    show_default=True,
-    multiple=True,
-    type=str,
-    help="""The model tasks to consider. If not specified then models will not be
-            filtered according to the task they were trained on.""",
+    specified. If "all" then the models will be benchmarked on all datasets. If not
+    specified then this will use the `language` value.""",
 )
 @click.option(
     "--dataset-task",
@@ -89,37 +79,35 @@ from .languages import get_all_languages
     help="The dataset tasks to consider.",
 )
 @click.option(
-    "--evaluate-train",
-    "-e",
-    is_flag=True,
+    "--evaluate-train/--no-evaluate-train",
+    default=False,
     show_default=True,
-    help="Whether the training set should be evaluated.",
+    help="Whether to evaluate the model on the training set.",
 )
 @click.option(
-    "--no-progress-bar",
-    "-np",
-    is_flag=True,
+    "--progress-bar/--no-progress-bar",
+    "-p/-np",
+    default=True,
     show_default=True,
-    help="Whether progress bars should be shown.",
+    help="Whether to show a progress bar.",
 )
 @click.option(
-    "--raise-error-on-invalid-model",
-    "-r",
-    is_flag=True,
+    "--raise-error-on-invalid-model/--no-raise-error-on-invalid-model",
+    default=False,
     show_default=True,
     help="Whether to raise an error if a model is invalid.",
 )
 @click.option(
-    "--verbose",
-    "-v",
-    is_flag=True,
+    "--verbose/--no-verbose",
+    "-v/-nv",
+    default=False,
     show_default=True,
     help="Whether extra input should be outputted during benchmarking",
 )
 @click.option(
-    "--no-save-results",
-    "-ns",
-    is_flag=True,
+    "--save-results/--no-save-results",
+    "-s/-ns",
+    default=True,
     show_default=True,
     help="Whether results should not be stored to disk.",
 )
@@ -131,12 +119,27 @@ from .languages import get_all_languages
     help="The directory where models are datasets are cached.",
 )
 @click.option(
-    "--use-auth-token",
-    is_flag=True,
+    "--auth-token",
+    type=str,
+    default="",
+    show_default=True,
+    help="""The authentication token for the Hugging Face Hub. If specified then the
+    `--use-auth-token` flag will be set to True.""",
+)
+@click.option(
+    "--use-auth-token/--no-use-auth-token",
+    default=False,
     show_default=True,
     help="""Whether an authentication token should be used, enabling evaluation of
-            private models. Requires that you are logged in via the
-            `huggingface-cli login` command.""",
+    private models. Requires that you are logged in via the `huggingface-cli login`
+    command.""",
+)
+@click.option(
+    "--ignore-duplicates/--no-ignore-duplicates",
+    default=True,
+    show_default=True,
+    help="""Whether to skip evaluation of models which have already been evaluated,
+    with scores lying in the 'scandeval_benchmark_results.jsonl' file.""",
 )
 def benchmark(
     model_id: Tuple[str],
@@ -145,15 +148,16 @@ def benchmark(
     model_language: Tuple[str],
     dataset_language: Tuple[str],
     raise_error_on_invalid_model: bool,
-    model_task: Tuple[str],
     dataset_task: Tuple[str],
     evaluate_train: bool,
-    no_progress_bar: bool,
-    no_save_results: bool,
+    progress_bar: bool,
+    save_results: bool,
     cache_dir: str,
     use_auth_token: bool,
+    auth_token: str,
+    ignore_duplicates: bool,
     verbose: bool = False,
-):
+) -> None:
     """Benchmark pretrained language models on Scandinavian language tasks."""
 
     # Set up language variables
@@ -162,22 +166,22 @@ def benchmark(
     languages: List[str] = list(language)
     model_languages = None if len(model_language) == 0 else list(model_language)
     dataset_languages = None if len(dataset_language) == 0 else list(dataset_language)
-    model_tasks = None if len(model_task) == 0 else list(model_task)
     dataset_tasks = None if len(dataset_task) == 0 else list(dataset_task)
+    auth: Union[str, bool] = auth_token if auth_token != "" else use_auth_token
 
     # Initialise the benchmarker class
     benchmarker = Benchmarker(
         language=languages,
         model_language=model_languages,
         dataset_language=dataset_languages,
-        model_task=model_tasks,
         dataset_task=dataset_tasks,
-        progress_bar=(not no_progress_bar),
-        save_results=(not no_save_results),
+        progress_bar=progress_bar,
+        save_results=save_results,
         evaluate_train=evaluate_train,
         raise_error_on_invalid_model=raise_error_on_invalid_model,
         verbose=verbose,
-        use_auth_token=use_auth_token,
+        use_auth_token=auth,
+        ignore_duplicates=ignore_duplicates,
         cache_dir=cache_dir,
     )
 
